#include "connectivitymanager.h"
#include <QString>
#include <QUrl>
#include <QDebug>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <QFile>
#include <QThread>

ConnectivityManager::ConnectivityManager() {}

ConnectivityManager::~ConnectivityManager() = default;

void ConnectivityManager::mountFtp(QString urlStr, QString userName, QString pwd, QString path)
{
    m_isMount = true;
    killMountProcess();
    QUrl url(urlStr);
    int uid = getuid();
    QString common = "curlftpfs -o rw,allow_other,uid=" + QString::number(uid) + " " + url.scheme() + "://" + userName
                     + ":" + pwd + "@" + url.host() + ":" + QString::number(url.port()) + " " + path;
    startCommand(common);
}

void ConnectivityManager::umountFtp(QString path)
{
    m_isMount = false;
    killMountProcess();
    QString common = "umount -f " + path;
    startCommand(common);
}

void ConnectivityManager::startCommand(QString common)
{
    qInfo() << common;
    QProcess *process = new QProcess(this);
    process->start(common);
    connect(process, &QProcess::readyReadStandardError, this, [=]() {
        QString errOutput = process->readAllStandardError();
        qInfo() << errOutput;
        if (!errOutput.isEmpty()) {
            Q_EMIT sigMountError(m_isMount, errOutput);
        }
        m_isMount = true;
        process->deleteLater();
    });
}

void ConnectivityManager::killProcess(const char *cmd)
{
    FILE *fp = popen(cmd, "r");
    if (fp == NULL) {
        qCritical() << "command fail !";
        return;
    }

    char buf[1024];
    memset(buf, '\0', sizeof(buf));
    while (!feof(fp)) {
        fgets(buf, sizeof(buf), fp);
        QString item(buf);
        memset(buf, '\0', sizeof(buf));

        if (item.isEmpty()) {
            continue;
        }

        if (item.contains("grep")) {
            continue;
        }
        QStringList itemList = item.trimmed().split(' ');
        itemList.removeAll("");
        QString info = itemList.value(1);
        bool ok = true;
        info.toLongLong(&ok);
        if (ok) {
            QString common = QString("kill -9 ") + info;
            QProcess::startDetached(common);
        }
    }

    pclose(fp);
}

void ConnectivityManager::killMountProcess()
{
    killProcess("ps -aux | /var/lib/mult/");
}

void ConnectivityManager::startDiscServer(QString cmd)
{
    qInfo() << "disc softbus init";
    m_discServer = new QProcess(this);
    connect(m_discServer, &QProcess::started, this, [=]() {
        Q_EMIT sigInitDiscServer(true);
    });
    connect(m_discServer, &QProcess::readyReadStandardError, this, [=]() {
        QString errOutput = m_discServer->readAllStandardError();
        qWarning() << "softbus server is not exist, softbus init failed." << errOutput;
        Q_EMIT sigInitDiscServer(false);
    });
    m_discServer->start(cmd);
}

void ConnectivityManager::stopDiscServer()
{
    qInfo() << "stop disc server";
    if (m_discServer) {
        if (m_discServer->state() == QProcess::Running) {
            m_discServer->close();
        }
        delete m_discServer;
        m_discServer = nullptr;
    }
}

void ConnectivityManager::killServerPortProcess()
{
    killProcess("lsof -i:27189");
}
