/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "curtain.h"

#include <QWebEngineView>
#include <QWebEngineSettings>
#include <QWebEngineFullScreenRequest>
#include <QApplication>
#include <QGuiApplication>
#include <QScreen>
#include <QDebug>
#include <QVBoxLayout>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

Curtain::Curtain(QWidget *parent) : QWidget(parent), m_view(new QWebEngineView(this))
{
    setWindowFlags(Qt::FramelessWindowHint); //无边框
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(m_view);
    layout->setMargin(0);
    layout->setSpacing(0);
    setLayout(layout);
    m_view->settings()->setAttribute(QWebEngineSettings::FullScreenSupportEnabled, true);
    m_view->setContextMenuPolicy(Qt::NoContextMenu);
    connect(m_view->page(), &QWebEnginePage::fullScreenRequested, this, &Curtain::fullScreenRequested);
    connect(m_view, &QWebEngineView::loadFinished, this, [=](bool bo) {
        qApp->setOverrideCursor(QCursor(Qt::BlankCursor));
        QString common = "xdotool mousemove 5 5 click 1";
        system(common.toUtf8().constData());
        qApp->setOverrideCursor(QCursor(Qt::ArrowCursor));
    });
}

Curtain::~Curtain() {}

void Curtain::showView()
{
    if (m_fullScreenWindow && m_fullScreenWindow->isHidden()) {
        m_fullScreenWindow->show();
        m_fullScreenWindow->showFullScreen();
        m_fullScreenWindow->activateWindow();
        m_fullScreenWindow->setFocus();
    }
    show();
}

void Curtain::hideView()
{
    if (m_fullScreenWindow && !m_fullScreenWindow->isHidden()) {
        m_fullScreenWindow->hide();
    }
    hide();
}

void Curtain::loadUrl(QString &url)
{
    qInfo() << "Sending request to projection service.";
    m_url = url;
    m_view->setUrl(QUrl(m_url));
}

void Curtain::fullScreenRequested(QWebEngineFullScreenRequest request)
{
    if (request.toggleOn()) {
        if (m_fullScreenWindow)
            return;
        request.accept();
        FullScreenWindow *fullScreenWindow = new FullScreenWindow(m_view);
        connect(fullScreenWindow, &FullScreenWindow::sigHideView, this, [=]() {
            hideView();
            Q_EMIT sigHideView();
        });
        m_fullScreenWindow.reset(fullScreenWindow);
    } else {
        if (!m_fullScreenWindow)
            return;
        request.accept();
        m_fullScreenWindow.reset();
    }
}