#include "mobilefilelist.h"
#include "mobilefilewidget.h"

#include <QDebug>

const QString ITEM_ICON_PATH = ":/icon/";
const QStringList ITEM_ICON_LIST = {"pictureIcon.png",  "videoIcon.png",  "musicIcon.png",
                                    "documentIcon.png", "weChatIcon.png", "qqIcon.png"};
const QStringList ITEM_NAME_LIST = {"picture", "video", "music", "document", "wechat", "qq"};
const int FILE_LIST_WIDGET_HEIGHT = 266;
const int STROAGE_WIDGET_HEIGHT = 175;
const int FILE_LIST_WIDGET_ITEM_WIDTH = 208;
const int FILE_LIST_WIDGET_ITEM_HEIGHT = 80;
const int STROAGE_WIDGET_ITEM_WIDTH = 208;
const int STROAGE_WIDGET_ITEM_HEIGHT = 80;

MobileFileList::MobileFileList(QWidget *parent) : QWidget(parent)
{
    initWidget();
    setItem();
    show();
}

MobileFileList::~MobileFileList() {}

void MobileFileList::setItemCount(const QMap<QString, int> &map)
{
    if (map.isEmpty()) {
        return;
    }
    QMap<QString, int>::const_iterator it;
    QMap<QString, MobileFileListItem *>::iterator itemIt = m_itemMap.begin();
    MobileFileListItem *item = nullptr;
    while (m_itemMap.end() != itemIt) {
        item = itemIt.value();
        it = map.find(itemIt.key());
        item->setItemCount(it.value());
        itemIt++;
    }
}

void MobileFileList::initWidget()
{
    QVBoxLayout *mainVLayout = new QVBoxLayout();

    QHBoxLayout *mobileFileListHLayout = new QHBoxLayout();
    QHBoxLayout *fileListWidgetHLayout = new QHBoxLayout();
    QHBoxLayout *mobileStorageHLayout = new QHBoxLayout();
    QHBoxLayout *storageWidgetHLayout = new QHBoxLayout();
    //手机文件列表
    QLabel *mobileFileListLabel = new QLabel();
    QFont font;
    font.setBold(true);
    mobileFileListLabel->setFont(font);
    mobileFileListLabel->setText(tr("Mobile file list")); // Mobile file list手机文件列表

    mobileFileListHLayout->addWidget(mobileFileListLabel);
    mobileFileListHLayout->setContentsMargins(34, 0, 0, 0);
    mobileFileListHLayout->addStretch();

    //手机存储
    QLabel *mobileStorageLabel = new QLabel();
    mobileStorageLabel->setFont(font);
    mobileStorageLabel->setText(tr("Mobile storage")); // Mobile storage移动存储

    mobileStorageHLayout->setMargin(0);
    mobileStorageHLayout->addWidget(mobileStorageLabel);
    mobileStorageHLayout->setContentsMargins(34, 0, 0, 0);
    mobileStorageHLayout->addStretch();

    //手机文件列表widget
    m_fileListWidget = new QListWidget();
    m_fileListWidget->setFixedHeight(FILE_LIST_WIDGET_HEIGHT);

    //设置显示图标模式
    m_fileListWidget->setViewMode(QListView::IconMode);
    //自适应布局
    m_fileListWidget->setResizeMode(QListView::Adjust);
    //列表中的图标不可拖动
    m_fileListWidget->setMovement(QListView::Static);
    //去边框
    m_fileListWidget->setFrameShape(QListWidget::NoFrame);
    //设置边距
    m_fileListWidget->setSpacing(16);
    //不可选择项目
    //    m_fileListWidget->setSelectionMode(QListView::NoSelection);

    QWidget *fileWidget = new QWidget();
    fileWidget->setLayout(fileListWidgetHLayout);
    fileWidget->setFixedHeight(FILE_LIST_WIDGET_HEIGHT);
    fileWidget->adjustSize();

    fileListWidgetHLayout->setContentsMargins(0, 0, 0, 0);
    fileListWidgetHLayout->addWidget(m_fileListWidget);

    //手机存储widget
    m_storageWidget = new QListWidget(this);

    QWidget *storageWidget = new QWidget();
    storageWidget->setLayout(storageWidgetHLayout);
    storageWidget->setFixedHeight(STROAGE_WIDGET_HEIGHT);

    storageWidgetHLayout->setContentsMargins(0, 0, 0, 0);
    storageWidgetHLayout->addWidget(m_storageWidget);
    storageWidgetHLayout->addStretch();

    //设置显示图标模式
    m_storageWidget->setViewMode(QListView::IconMode);
    //自适应布局
    m_storageWidget->setResizeMode(QListView::Adjust);
    //列表中的图标不可拖动
    m_storageWidget->setMovement(QListView::Static);
    //去边框
    m_storageWidget->setFrameShape(QListWidget::NoFrame);
    //设置边距
    m_storageWidget->setSpacing(16);

    mainVLayout->setContentsMargins(56, 0, 0, 0);
    mainVLayout->setSpacing(0);
    mainVLayout->addLayout(mobileFileListHLayout);
    mainVLayout->addSpacing(28);
    mainVLayout->addWidget(fileWidget);
    mainVLayout->addSpacing(0);
    mainVLayout->addLayout(mobileStorageHLayout);
    mainVLayout->addSpacing(28);
    mainVLayout->addWidget(storageWidget);
    mainVLayout->addStretch();
    this->setLayout(mainVLayout);
}

void MobileFileList::setItem()
{
    for (int i = 0; i < 6; i++) {
        QListWidgetItem *item = new QListWidgetItem(m_fileListWidget);
        MobileFileListItem *mobileFileListItem = new MobileFileListItem();
        m_fileListWidget->setItemWidget(item, mobileFileListItem);
        item->setSizeHint(QSize(FILE_LIST_WIDGET_ITEM_WIDTH, FILE_LIST_WIDGET_ITEM_HEIGHT));
        mobileFileListItem->setFocusPolicy(Qt::NoFocus);
        mobileFileListItem->setItemCount(0);
        mobileFileListItem->setFixedSize(208, 80);
        mobileFileListItem->setItemIcon(ITEM_ICON_PATH + ITEM_ICON_LIST.value(i));
        m_itemMap.insert(ITEM_NAME_LIST.value(i), mobileFileListItem);
        switch (i) {
        case 0:
            mobileFileListItem->setItemName(tr("Picture")); // Picture图片
            connect(mobileFileListItem, &MobileFileListItem::clicked, this, &MobileFileList::slotPictureButtonClicked);
            break;
        case 1:
            mobileFileListItem->setItemName(tr("Video")); // Video视频
            connect(mobileFileListItem, &MobileFileListItem::clicked, this, &MobileFileList::slotVideoButtonClicked);
            break;
        case 2:
            mobileFileListItem->setItemName(tr("Music")); // Music音乐
            connect(mobileFileListItem, &MobileFileListItem::clicked, this, &MobileFileList::slotMusicButtonClicked);
            break;
        case 3:
            mobileFileListItem->setItemName(tr("Document")); // Document文档
            connect(mobileFileListItem, &MobileFileListItem::clicked, this, &MobileFileList::slotDocumentButtonClicked);
            break;
        case 4:
            mobileFileListItem->setItemName(tr("WeChat")); // WeChat微信
            connect(mobileFileListItem, &MobileFileListItem::clicked, this, &MobileFileList::slotWeChatButtonClicked);
            break;
        case 5:
            mobileFileListItem->setItemName(tr("QQ")); // QQ   QQ
            connect(mobileFileListItem, &MobileFileListItem::clicked, this, &MobileFileList::slotQQButtonClicked);
            break;
        default:
            break;
        }
    }
    QListWidgetItem *item = new QListWidgetItem(m_storageWidget);
    MobileFileListItem *mobileFileListItem = new MobileFileListItem();
    m_storageWidget->setItemWidget(item, mobileFileListItem);
    item->setSizeHint(QSize(STROAGE_WIDGET_ITEM_WIDTH, STROAGE_WIDGET_ITEM_HEIGHT));
    mobileFileListItem->setItemName(tr("Mobile"));  // Mobile手机
    mobileFileListItem->setAllFile(tr("All File")); // All File所有文件
    mobileFileListItem->setFixedSize(208, 80);
    mobileFileListItem->setItemIcon(":/icon/mobileFileIcon.png");
    connect(mobileFileListItem, &MobileFileListItem::clicked, this, &MobileFileList::slotMobileButtonClicked);
}

void MobileFileList::slotPictureButtonClicked()
{
    Q_EMIT sigPictureClicked(FILE_LIST_TYPE::PICTURE);
}

void MobileFileList::slotVideoButtonClicked()
{
    Q_EMIT sigVideoClicked(FILE_LIST_TYPE::VIDEO);
}

void MobileFileList::slotMusicButtonClicked()
{
    Q_EMIT sigMusicClicked(FILE_LIST_TYPE::MUSIC);
}

void MobileFileList::slotDocumentButtonClicked()
{
    Q_EMIT sigDocumentClicked(FILE_LIST_TYPE::DOCUMENT);
}

void MobileFileList::slotWeChatButtonClicked()
{
    Q_EMIT sigWeChatClicked(FILE_LIST_TYPE::WECHAT);
}

void MobileFileList::slotQQButtonClicked()
{
    Q_EMIT sigQQClicked(FILE_LIST_TYPE::QQ);
}

void MobileFileList::slotMobileButtonClicked()
{
    Q_EMIT sigMobileStorageClicked("/");
}

void MobileFileList::setTheme(PublicAttributes::Theme theme)
{
    if (theme == PublicAttributes::Theme::Light) {
        QPalette paletteListWid = m_fileListWidget->palette();
        paletteListWid.setColor(QPalette::Base, QColor("#FFFEFD"));
        m_fileListWidget->setPalette(paletteListWid);
        QPalette paletteStorageWid = m_storageWidget->palette();
        paletteStorageWid.setColor(QPalette::Base, QColor("#FFFEFD"));
        m_storageWidget->setPalette(paletteStorageWid);
    } else if (theme == PublicAttributes::Theme::Dark) {
        QPalette paletteListWid = m_fileListWidget->palette();
        paletteListWid.setColor(QPalette::Base, QColor("#1D1D1D"));
        m_fileListWidget->setPalette(paletteListWid);
        QPalette palette = m_storageWidget->palette();
        palette.setColor(QPalette::Base, QColor("#1D1D1D"));
        m_storageWidget->setPalette(palette);
    }
}

